@@ -9,11 +9,9 @@ Python Scheduler exists.
 # Distributed under the terms of the Modified BSD License.
 
 import logging
-import sys
 import time
 
 from collections import deque
-from datetime import datetime
 from random import randint, random
 from types import FunctionType
 
@@ -23,7 +21,7 @@ except ImportError:
     numpy = None
 
 import zmq
-from zmq.eventloop import ioloop, zmqstream
+from zmq.eventloop import zmqstream
 
 # local imports
 from decorator import decorator
@@ -34,7 +32,7 @@ from ipython_genutils.py3compat import cast_bytes
 
 from ipyparallel import error, util
 from ipyparallel.factory import SessionFactory
-from ipyparallel.util import connect_logger, local_logger
+from ipyparallel.util import connect_logger, local_logger, ioloop
 
 from .dependency import Dependency
 
@@ -321,24 +319,24 @@ help="""select the task scheduler scheme  [default: Python LRU]
         # wait 5 seconds before cleaning up pending jobs, since the results might
         # still be incoming
         if self.pending[uid]:
-            self.loop.add_timeout(self.loop.time() + 5,
-                lambda : self.handle_stranded_tasks(uid),
+            self.loop.add_timeout(
+                self.loop.time() + 5,
+                lambda: self.handle_stranded_tasks(uid),
             )
         else:
             self.completed.pop(uid)
             self.failed.pop(uid)
 
-
     def handle_stranded_tasks(self, engine):
         """Deal with jobs resident in an engine that died."""
         lost = self.pending[engine]
-        for msg_id in lost.keys():
-            if msg_id not in self.pending[engine]:
+        for msg_id in list(lost.keys()):
+            if msg_id not in lost:
                 # prevent double-handling of messages
                 continue
 
             raw_msg = lost[msg_id].raw_msg
-            idents,msg = self.session.feed_identities(raw_msg, copy=False)
+            idents, msg = self.session.feed_identities(raw_msg, copy=False)
             parent = self.session.unpack(msg[1].bytes)
             idents = [engine, idents[0]]
 
@@ -749,7 +747,7 @@ help="""select the task scheduler scheme  [default: Python LRU]
                     for mid in job.dependents:
                         if mid in self.graph:
                             self.graph[mid].remove(msg_id)
-                    
+
                     # abort the loop if we just filled up all of our engines.
                     # avoids an O(N) operation in situation of full queue,
                     # where graph update is triggered as soon as an engine becomes
@@ -757,17 +755,17 @@ help="""select the task scheduler scheme  [default: Python LRU]
                     # even though they can't run.
                     if not self.available_engines():
                         break
-            
+
             if using_queue and put_it_back:
                 # popped a job from the queue but it neither ran nor failed,
                 # so we need to put it back when we are done
                 # make sure to_restore preserves the same ordering
                 to_restore.append(job)
-        
+
         # put back any tasks we popped but didn't run
         if using_queue:
             self.queue.extendleft(to_restore)
-    
+
     #----------------------------------------------------------------------
     # methods to be overridden by subclasses
     #----------------------------------------------------------------------
@@ -780,14 +778,12 @@ help="""select the task scheduler scheme  [default: Python LRU]
         for lis in (self.targets, self.loads):
             lis.append(lis.pop(idx))
 
-
     def finish_job(self, idx):
         """Called after self.targets[idx] just finished a job.
         Override with subclasses."""
         self.loads[idx] -= 1
 
 
-
 def launch_scheduler(in_addr, out_addr, mon_addr, not_addr, reg_addr, config=None,
                         logname='root', log_url=None, loglevel=logging.DEBUG,
                         identity=b'task', in_thread=False):
@@ -801,7 +797,7 @@ def launch_scheduler(in_addr, out_addr, mon_addr, not_addr, reg_addr, config=Non
     if in_thread:
         # use instance() to get the same Context/Loop as our parent
         ctx = zmq.Context.instance()
-        loop = ioloop.IOLoop.instance()
+        loop = ioloop.IOLoop.current()
     else:
         # in a process, don't use instance()
         # for safety with multiprocessing
@@ -822,10 +818,10 @@ def launch_scheduler(in_addr, out_addr, mon_addr, not_addr, reg_addr, config=Non
     nots = zmqstream.ZMQStream(ctx.socket(zmq.SUB),loop)
     nots.setsockopt(zmq.SUBSCRIBE, b'')
     nots.connect(not_addr)
-    
+
     querys = ZMQStream(ctx.socket(zmq.DEALER),loop)
     querys.connect(reg_addr)
-    
+
     # setup logging.
     if in_thread:
         log = Application.instance().log
