@@ -4,6 +4,16 @@
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
+try:
+    from functools import lru_cache
+except ImportError:
+    # py2 has no lru_cache decorator,
+    # use a no-op
+    def lru_cache(maxsize=0):
+        """no-op decorator when lru_cache is unavailable"""
+        return lambda f: f
+
+from distutils.version import LooseVersion as V
 import logging
 import os
 import re
@@ -16,7 +26,7 @@ from signal import signal, SIGINT, SIGABRT, SIGTERM
 try:
     from signal import SIGKILL
 except ImportError:
-    SIGKILL=None
+    SIGKILL = None
 from types import FunctionType
 
 from dateutil.parser import parse as dateutil_parse
@@ -34,6 +44,11 @@ from tornado.ioloop import IOLoop
 import zmq
 from zmq.log import handlers
 
+if V(zmq.__version__) < V('17.0'):
+    from zmq.eventloop import ioloop
+else:
+    from tornado import ioloop
+
 from traitlets.log import get_logger
 
 
@@ -177,6 +192,29 @@ def split_url(url):
     return proto,addr,s_port
 
 
+def is_ip(location):
+    """Is a location an ip?
+    
+    It could be a hostname.
+    """
+    return bool(re.match(location, '(\d+\.){3}\d+'))
+
+
+@lru_cache()
+def ip_for_host(host):
+    """Get the ip address for a host
+    
+    If no ips can be found for the host,
+    the host is returned unmodified.
+    """
+    try:
+        return socket.gethostbyname_ex(host)[2][0]
+    except Exception as e:
+        warnings.warn("IPython could not determine IPs for %s: %s" % (host, e),
+            RuntimeWarning)
+        return host
+
+
 def disambiguate_ip_address(ip, location=None):
     """turn multi-ip interfaces '0.0.0.0' and '*' into a connectable address
     
@@ -187,8 +225,8 @@ def disambiguate_ip_address(ip, location=None):
     
     ip : IP address
         An IP address, or the special values 0.0.0.0, or *
-    location: IP address, optional
-        A public IP of the target machine.
+    location: IP address or hostname, optional
+        A public IP of the target machine, or its hostname.
         If location is an IP of the current machine,
         localhost will be returned,
         otherwise location will be returned.
@@ -196,8 +234,16 @@ def disambiguate_ip_address(ip, location=None):
     if ip in {'0.0.0.0', '*'}:
         if not location:
             # unspecified location, localhost is the only choice
-            ip = localhost()
-        elif is_public_ip(location):
+            return localhost()
+        elif not is_ip(location):
+            if location == socket.gethostname():
+                # hostname matches, use localhost
+                return localhost()
+            else:
+                # hostname doesn't match, but the machine can have a few names.
+                location = ip_for_host(location)
+
+        if is_public_ip(location):
             # location is a public IP on this machine, use localhost
             ip = localhost()
         elif not public_ips():
